@@ -3,8 +3,8 @@ package nghttp2
 import (
 	"crypto/tls"
 	"fmt"
-	"github.com/bradfitz/http2"
-	"github.com/bradfitz/http2/hpack"
+	"golang.org/x/net/http2"
+	"golang.org/x/net/http2/hpack"
 	"io"
 	"io/ioutil"
 	"net/http"
@@ -644,7 +644,7 @@ func TestH2H1HeaderFields(t *testing.T) {
 // TestH2H1ReqPhaseSetHeader tests mruby request phase hook
 // modifies request header fields.
 func TestH2H1ReqPhaseSetHeader(t *testing.T) {
-	st := newServerTester([]string{"--request-phase-file=" + testDir + "/req-set-header.rb"}, t, func(w http.ResponseWriter, r *http.Request) {
+	st := newServerTester([]string{"--mruby-file=" + testDir + "/req-set-header.rb"}, t, func(w http.ResponseWriter, r *http.Request) {
 		if got, want := r.Header.Get("User-Agent"), "mruby"; got != want {
 			t.Errorf("User-Agent = %v; want %v", got, want)
 		}
@@ -666,7 +666,7 @@ func TestH2H1ReqPhaseSetHeader(t *testing.T) {
 // TestH2H1ReqPhaseReturn tests mruby request phase hook returns
 // custom response.
 func TestH2H1ReqPhaseReturn(t *testing.T) {
-	st := newServerTester([]string{"--request-phase-file=" + testDir + "/return.rb"}, t, func(w http.ResponseWriter, r *http.Request) {
+	st := newServerTester([]string{"--mruby-file=" + testDir + "/req-return.rb"}, t, func(w http.ResponseWriter, r *http.Request) {
 		t.Fatalf("request should not be forwarded")
 	})
 	defer st.Close()
@@ -685,7 +685,7 @@ func TestH2H1ReqPhaseReturn(t *testing.T) {
 	hdtests := []struct {
 		k, v string
 	}{
-		{"content-length", "11"},
+		{"content-length", "20"},
 		{"from", "mruby"},
 	}
 	for _, tt := range hdtests {
@@ -694,7 +694,7 @@ func TestH2H1ReqPhaseReturn(t *testing.T) {
 		}
 	}
 
-	if got, want := string(res.body), "Hello World"; got != want {
+	if got, want := string(res.body), "Hello World from req"; got != want {
 		t.Errorf("body = %v; want %v", got, want)
 	}
 }
@@ -702,7 +702,7 @@ func TestH2H1ReqPhaseReturn(t *testing.T) {
 // TestH2H1RespPhaseSetHeader tests mruby response phase hook modifies
 // response header fields.
 func TestH2H1RespPhaseSetHeader(t *testing.T) {
-	st := newServerTester([]string{"--response-phase-file=" + testDir + "/resp-set-header.rb"}, t, noopHandler)
+	st := newServerTester([]string{"--mruby-file=" + testDir + "/resp-set-header.rb"}, t, noopHandler)
 	defer st.Close()
 
 	res, err := st.http2(requestParam{
@@ -724,7 +724,7 @@ func TestH2H1RespPhaseSetHeader(t *testing.T) {
 // TestH2H1RespPhaseReturn tests mruby response phase hook returns
 // custom response.
 func TestH2H1RespPhaseReturn(t *testing.T) {
-	st := newServerTester([]string{"--response-phase-file=" + testDir + "/return.rb"}, t, noopHandler)
+	st := newServerTester([]string{"--mruby-file=" + testDir + "/resp-return.rb"}, t, noopHandler)
 	defer st.Close()
 
 	res, err := st.http2(requestParam{
@@ -741,7 +741,7 @@ func TestH2H1RespPhaseReturn(t *testing.T) {
 	hdtests := []struct {
 		k, v string
 	}{
-		{"content-length", "11"},
+		{"content-length", "21"},
 		{"from", "mruby"},
 	}
 	for _, tt := range hdtests {
@@ -750,7 +750,7 @@ func TestH2H1RespPhaseReturn(t *testing.T) {
 		}
 	}
 
-	if got, want := string(res.body), "Hello World"; got != want {
+	if got, want := string(res.body), "Hello World from resp"; got != want {
 		t.Errorf("body = %v; want %v", got, want)
 	}
 }
@@ -1351,7 +1351,7 @@ func TestH2H2TLSXfp(t *testing.T) {
 // TestH2H2ReqPhaseReturn tests mruby request phase hook returns
 // custom response.
 func TestH2H2ReqPhaseReturn(t *testing.T) {
-	st := newServerTester([]string{"--http2-bridge", "--request-phase-file=" + testDir + "/return.rb"}, t, func(w http.ResponseWriter, r *http.Request) {
+	st := newServerTester([]string{"--http2-bridge", "--mruby-file=" + testDir + "/req-return.rb"}, t, func(w http.ResponseWriter, r *http.Request) {
 		t.Fatalf("request should not be forwarded")
 	})
 	defer st.Close()
@@ -1370,7 +1370,7 @@ func TestH2H2ReqPhaseReturn(t *testing.T) {
 	hdtests := []struct {
 		k, v string
 	}{
-		{"content-length", "11"},
+		{"content-length", "20"},
 		{"from", "mruby"},
 	}
 	for _, tt := range hdtests {
@@ -1379,7 +1379,7 @@ func TestH2H2ReqPhaseReturn(t *testing.T) {
 		}
 	}
 
-	if got, want := string(res.body), "Hello World"; got != want {
+	if got, want := string(res.body), "Hello World from req"; got != want {
 		t.Errorf("body = %v; want %v", got, want)
 	}
 }
@@ -1387,7 +1387,7 @@ func TestH2H2ReqPhaseReturn(t *testing.T) {
 // TestH2H2RespPhaseReturn tests mruby response phase hook returns
 // custom response.
 func TestH2H2RespPhaseReturn(t *testing.T) {
-	st := newServerTester([]string{"--http2-bridge", "--response-phase-file=" + testDir + "/return.rb"}, t, noopHandler)
+	st := newServerTester([]string{"--http2-bridge", "--mruby-file=" + testDir + "/resp-return.rb"}, t, noopHandler)
 	defer st.Close()
 
 	res, err := st.http2(requestParam{
@@ -1404,7 +1404,7 @@ func TestH2H2RespPhaseReturn(t *testing.T) {
 	hdtests := []struct {
 		k, v string
 	}{
-		{"content-length", "11"},
+		{"content-length", "21"},
 		{"from", "mruby"},
 	}
 	for _, tt := range hdtests {
@@ -1413,7 +1413,7 @@ func TestH2H2RespPhaseReturn(t *testing.T) {
 		}
 	}
 
-	if got, want := string(res.body), "Hello World"; got != want {
+	if got, want := string(res.body), "Hello World from resp"; got != want {
 		t.Errorf("body = %v; want %v", got, want)
 	}
 }
