@@ -41,6 +41,7 @@
 #include <openssl/x509.h>
 #include <openssl/x509v3.h>
 #include <openssl/rand.h>
+#include <openssl/dh.h>
 
 #include <nghttp2/nghttp2.h>
 
@@ -153,6 +154,7 @@ int servername_callback(SSL *ssl, int *al, void *arg) {
 }
 } // namespace
 
+#ifndef OPENSSL_IS_BORINGSSL
 namespace {
 std::shared_ptr<std::vector<uint8_t>>
 get_ocsp_data(TLSContextData *tls_ctx_data) {
@@ -187,6 +189,7 @@ int ocsp_resp_cb(SSL *ssl, void *arg) {
   return SSL_TLSEXT_ERR_OK;
 }
 } // namespace
+#endif // OPENSSL_IS_BORINGSSL
 
 constexpr char MEMCACHED_SESSION_CACHE_KEY_PREFIX[] =
     "nghttpx:tls-session-cache:";
@@ -456,8 +459,12 @@ long int create_tls_proto_mask(const std::vector<std::string> &tls_proto_list) {
   return res;
 }
 
-SSL_CTX *create_ssl_context(const char *private_key_file,
-                            const char *cert_file) {
+SSL_CTX *create_ssl_context(const char *private_key_file, const char *cert_file
+#ifdef HAVE_NEVERBLEED
+                            ,
+                            neverbleed_t *nb
+#endif // HAVE_NEVERBLEED
+                            ) {
   auto ssl_ctx = SSL_CTX_new(SSLv23_server_method());
   if (!ssl_ctx) {
     LOG(FATAL) << ERR_error_string(ERR_get_error(), nullptr);
@@ -543,12 +550,22 @@ SSL_CTX *create_ssl_context(const char *private_key_file,
     SSL_CTX_set_default_passwd_cb(ssl_ctx, ssl_pem_passwd_cb);
     SSL_CTX_set_default_passwd_cb_userdata(ssl_ctx, (void *)get_config());
   }
+
+#ifndef HAVE_NEVERBLEED
   if (SSL_CTX_use_PrivateKey_file(ssl_ctx, private_key_file,
                                   SSL_FILETYPE_PEM) != 1) {
     LOG(FATAL) << "SSL_CTX_use_PrivateKey_file failed: "
                << ERR_error_string(ERR_get_error(), nullptr);
+  }
+#else  // HAVE_NEVERBLEED
+  std::array<char, NEVERBLEED_ERRBUF_SIZE> errbuf;
+  if (neverbleed_load_private_key_file(nb, ssl_ctx, private_key_file,
+                                       errbuf.data()) != 1) {
+    LOG(FATAL) << "neverbleed_load_private_key_file failed: " << errbuf.data();
     DIE();
   }
+#endif // HAVE_NEVERBLEED
+
   if (SSL_CTX_use_certificate_chain_file(ssl_ctx, cert_file) != 1) {
     LOG(FATAL) << "SSL_CTX_use_certificate_file failed: "
                << ERR_error_string(ERR_get_error(), nullptr);
@@ -590,7 +607,9 @@ SSL_CTX *create_ssl_context(const char *private_key_file,
   }
   SSL_CTX_set_tlsext_servername_callback(ssl_ctx, servername_callback);
   SSL_CTX_set_tlsext_ticket_key_cb(ssl_ctx, ticket_key_cb);
+#ifndef OPENSSL_IS_BORINGSSL
   SSL_CTX_set_tlsext_status_cb(ssl_ctx, ocsp_resp_cb);
+#endif // OPENSSL_IS_BORINGSSL
   SSL_CTX_set_info_callback(ssl_ctx, info_callback);
 
   // NPN advertisement
@@ -621,7 +640,11 @@ int select_next_proto_cb(SSL *ssl, unsigned char **out, unsigned char *outlen,
 }
 } // namespace
 
-SSL_CTX *create_ssl_client_context() {
+SSL_CTX *create_ssl_client_context(
+#ifdef HAVE_NEVERBLEED
+    neverbleed_t *nb
+#endif // HAVE_NEVERBLEED
+    ) {
   auto ssl_ctx = SSL_CTX_new(SSLv23_client_method());
   if (!ssl_ctx) {
     LOG(FATAL) << ERR_error_string(ERR_get_error(), nullptr);
@@ -667,6 +690,7 @@ SSL_CTX *create_ssl_client_context() {
   }
 
   if (get_config()->client_private_key_file) {
+#ifndef HAVE_NEVERBLEED
     if (SSL_CTX_use_PrivateKey_file(ssl_ctx,
                                     get_config()->client_private_key_file.get(),
                                     SSL_FILETYPE_PEM) != 1) {
@@ -675,6 +699,16 @@ SSL_CTX *create_ssl_client_context() {
                  << ERR_error_string(ERR_get_error(), nullptr);
       DIE();
     }
+#else  // HAVE_NEVERBLEED
+    std::array<char, NEVERBLEED_ERRBUF_SIZE> errbuf;
+    if (neverbleed_load_private_key_file(
+            nb, ssl_ctx, get_config()->client_private_key_file.get(),
+            errbuf.data()) != 1) {
+      LOG(FATAL) << "neverbleed_load_private_key_file failed: "
+                 << errbuf.data();
+      DIE();
+    }
+#endif // HAVE_NEVERBLEED
   }
   if (get_config()->client_cert_file) {
     if (SSL_CTX_use_certificate_chain_file(
@@ -759,7 +793,7 @@ bool tls_hostname_match(const char *pattern, const char *hostname) {
   // Don't attempt to match a presented identifier where the wildcard
   // character is embedded within an A-label.
   if (ptLeftLabelEnd == 0 || strchr(ptLeftLabelEnd + 1, '.') == 0 ||
-      ptLeftLabelEnd < ptWildcard || util::istartsWith(pattern, "xn--")) {
+      ptLeftLabelEnd < ptWildcard || util::istarts_with(pattern, "xn--")) {
     wildcardEnabled = false;
   }
   if (!wildcardEnabled) {
@@ -774,9 +808,9 @@ bool tls_hostname_match(const char *pattern, const char *hostname) {
   if (hnLeftLabelEnd - hostname < ptLeftLabelEnd - pattern) {
     return false;
   }
-  return util::istartsWith(hostname, hnLeftLabelEnd, pattern, ptWildcard) &&
-         util::iendsWith(hostname, hnLeftLabelEnd, ptWildcard + 1,
-                         ptLeftLabelEnd);
+  return util::istarts_with(hostname, hnLeftLabelEnd, pattern, ptWildcard) &&
+         util::iends_with(hostname, hnLeftLabelEnd, ptWildcard + 1,
+                          ptLeftLabelEnd);
 }
 } // namespace
 
@@ -897,7 +931,10 @@ int check_cert(SSL *ssl, const DownstreamAddr *addr) {
   std::vector<std::string> dns_names;
   std::vector<std::string> ip_addrs;
   get_altnames(cert, dns_names, ip_addrs, common_name);
-  if (verify_hostname(addr->host.get(), &addr->addr, dns_names, ip_addrs,
+  auto hostname = get_config()->backend_tls_sni_name
+                      ? get_config()->backend_tls_sni_name.get()
+                      : addr->host.get();
+  if (verify_hostname(hostname, &addr->addr, dns_names, ip_addrs,
                       common_name) != 0) {
     LOG(ERROR) << "Certificate verification failed: hostname does not match";
     return -1;
@@ -1096,13 +1133,23 @@ bool in_proto_list(const std::vector<std::string> &protos,
 }
 
 SSL_CTX *setup_server_ssl_context(std::vector<SSL_CTX *> &all_ssl_ctx,
-                                  CertLookupTree *cert_tree) {
+                                  CertLookupTree *cert_tree
+#ifdef HAVE_NEVERBLEED
+                                  ,
+                                  neverbleed_t *nb
+#endif // HAVE_NEVERBLEED
+                                  ) {
   if (get_config()->upstream_no_tls) {
     return nullptr;
   }
 
   auto ssl_ctx = ssl::create_ssl_context(get_config()->private_key_file.get(),
-                                         get_config()->cert_file.get());
+                                         get_config()->cert_file.get()
+#ifdef HAVE_NEVERBLEED
+                                             ,
+                                         nb
+#endif // HAVE_NEVERBLEED
+                                         );
 
   all_ssl_ctx.push_back(ssl_ctx);
 
@@ -1118,7 +1165,12 @@ SSL_CTX *setup_server_ssl_context(std::vector<SSL_CTX *> &all_ssl_ctx,
 
   for (auto &keycert : get_config()->subcerts) {
     auto ssl_ctx =
-        ssl::create_ssl_context(keycert.first.c_str(), keycert.second.c_str());
+        ssl::create_ssl_context(keycert.first.c_str(), keycert.second.c_str()
+#ifdef HAVE_NEVERBLEED
+                                                           ,
+                                nb
+#endif // HAVE_NEVERBLEED
+                                );
     all_ssl_ctx.push_back(ssl_ctx);
     if (ssl::cert_lookup_tree_add_cert_from_file(
             cert_tree, ssl_ctx, keycert.second.c_str()) == -1) {
@@ -1136,15 +1188,28 @@ SSL_CTX *setup_server_ssl_context(std::vector<SSL_CTX *> &all_ssl_ctx,
   return ssl_ctx;
 }
 
-SSL_CTX *setup_client_ssl_context() {
+bool downstream_tls_enabled() {
   if (get_config()->client_mode) {
-    return get_config()->downstream_no_tls ? nullptr
-                                           : ssl::create_ssl_client_context();
+    return !get_config()->downstream_no_tls;
   }
 
-  return get_config()->http2_bridge && !get_config()->downstream_no_tls
-             ? ssl::create_ssl_client_context()
-             : nullptr;
+  return get_config()->http2_bridge && !get_config()->downstream_no_tls;
+}
+
+SSL_CTX *setup_client_ssl_context(
+#ifdef HAVE_NEVERBLEED
+    neverbleed_t *nb
+#endif // HAVE_NEVERBLEED
+    ) {
+  if (!downstream_tls_enabled()) {
+    return nullptr;
+  }
+
+  return ssl::create_ssl_client_context(
+#ifdef HAVE_NEVERBLEED
+      nb
+#endif // HAVE_NEVERBLEED
+      );
 }
 
 CertLookupTree *create_cert_lookup_tree() {
