@@ -44,6 +44,7 @@
 #include <sstream>
 #include <memory>
 #include <chrono>
+#include <map>
 
 #include "http-parser/http_parser.h"
 
@@ -63,115 +64,17 @@ constexpr const char NGHTTP2_H1_1[] = "http/1.1";
 
 namespace util {
 
-extern const char DEFAULT_STRIP_CHARSET[];
-
-template <typename InputIterator>
-std::pair<InputIterator, InputIterator>
-stripIter(InputIterator first, InputIterator last,
-          const char *chars = DEFAULT_STRIP_CHARSET) {
-  for (; first != last && strchr(chars, *first) != 0; ++first)
-    ;
-  if (first == last) {
-    return std::make_pair(first, last);
-  }
-  InputIterator left = last - 1;
-  for (; left != first && strchr(chars, *left) != 0; --left)
-    ;
-  return std::make_pair(first, left + 1);
-}
-
-template <typename InputIterator, typename OutputIterator>
-OutputIterator splitIter(InputIterator first, InputIterator last,
-                         OutputIterator out, char delim, bool doStrip = false,
-                         bool allowEmpty = false) {
-  for (InputIterator i = first; i != last;) {
-    InputIterator j = std::find(i, last, delim);
-    std::pair<InputIterator, InputIterator> p(i, j);
-    if (doStrip) {
-      p = stripIter(i, j);
-    }
-    if (allowEmpty || p.first != p.second) {
-      *out++ = p;
-    }
-    i = j;
-    if (j != last) {
-      ++i;
-    }
-  }
-  if (allowEmpty && (first == last || *(last - 1) == delim)) {
-    *out++ = std::make_pair(last, last);
-  }
-  return out;
-}
-
-template <typename InputIterator, typename OutputIterator>
-OutputIterator split(InputIterator first, InputIterator last,
-                     OutputIterator out, char delim, bool doStrip = false,
-                     bool allowEmpty = false) {
-  for (InputIterator i = first; i != last;) {
-    InputIterator j = std::find(i, last, delim);
-    std::pair<InputIterator, InputIterator> p(i, j);
-    if (doStrip) {
-      p = stripIter(i, j);
-    }
-    if (allowEmpty || p.first != p.second) {
-      *out++ = std::string(p.first, p.second);
-    }
-    i = j;
-    if (j != last) {
-      ++i;
-    }
-  }
-  if (allowEmpty && (first == last || *(last - 1) == delim)) {
-    *out++ = std::string(last, last);
-  }
-  return out;
-}
-
-template <typename InputIterator, typename DelimiterType>
-std::string strjoin(InputIterator first, InputIterator last,
-                    const DelimiterType &delim) {
-  std::string result;
-  if (first == last) {
-    return result;
-  }
-  InputIterator beforeLast = last - 1;
-  for (; first != beforeLast; ++first) {
-    result += *first;
-    result += delim;
-  }
-  result += *beforeLast;
-  return result;
-}
-
-template <typename InputIterator>
-std::string joinPath(InputIterator first, InputIterator last) {
-  std::vector<std::string> elements;
-  for (; first != last; ++first) {
-    if (*first == "..") {
-      if (!elements.empty()) {
-        elements.pop_back();
-      }
-    } else if (*first == ".") {
-      // do nothing
-    } else {
-      elements.push_back(*first);
-    }
-  }
-  return strjoin(elements.begin(), elements.end(), "/");
-}
-
-inline bool isAlpha(const char c) {
+inline bool is_alpha(const char c) {
   return ('A' <= c && c <= 'Z') || ('a' <= c && c <= 'z');
 }
 
-inline bool isDigit(const char c) { return '0' <= c && c <= '9'; }
+inline bool is_digit(const char c) { return '0' <= c && c <= '9'; }
 
-inline bool isHexDigit(const char c) {
-  return isDigit(c) || ('A' <= c && c <= 'F') || ('a' <= c && c <= 'f');
+inline bool is_hex_digit(const char c) {
+  return is_digit(c) || ('A' <= c && c <= 'F') || ('a' <= c && c <= 'f');
 }
 
-bool inRFC3986UnreservedChars(const char c);
+bool in_rfc3986_unreserved_chars(const char c);
 
 bool in_rfc3986_sub_delims(const char c);
 
@@ -181,23 +84,23 @@ bool in_token(char c);
 bool in_attr_char(char c);
 
 // Returns integer corresponding to hex notation |c|.  It is undefined
-// if isHexDigit(c) is false.
+// if is_hex_digit(c) is false.
 uint32_t hex_to_uint(char c);
 
-std::string percentEncode(const unsigned char *target, size_t len);
+std::string percent_encode(const unsigned char *target, size_t len);
 
-std::string percentEncode(const std::string &target);
+std::string percent_encode(const std::string &target);
 
 // percent-encode path component of URI |s|.
 std::string percent_encode_path(const std::string &s);
 
 template <typename InputIt>
-std::string percentDecode(InputIt first, InputIt last) {
+std::string percent_decode(InputIt first, InputIt last) {
   std::string result;
   for (; first != last; ++first) {
     if (*first == '%') {
-      if (first + 1 != last && first + 2 != last && isHexDigit(*(first + 1)) &&
-          isHexDigit(*(first + 2))) {
+      if (first + 1 != last && first + 2 != last &&
+          is_hex_digit(*(first + 1)) && is_hex_digit(*(first + 2))) {
         result += (hex_to_uint(*(first + 1)) << 4) + hex_to_uint(*(first + 2));
         first += 2;
         continue;
@@ -219,7 +122,7 @@ std::string quote_string(const std::string &target);
 
 std::string format_hex(const unsigned char *s, size_t len);
 
-template <size_t N> std::string format_hex(const unsigned char (&s)[N]) {
+template <size_t N> std::string format_hex(const unsigned char(&s)[N]) {
   return format_hex(s, N);
 }
 
@@ -266,20 +169,20 @@ inline char lowcase(char c) {
 }
 
 template <typename InputIterator1, typename InputIterator2>
-bool startsWith(InputIterator1 first1, InputIterator1 last1,
-                InputIterator2 first2, InputIterator2 last2) {
+bool starts_with(InputIterator1 first1, InputIterator1 last1,
+                 InputIterator2 first2, InputIterator2 last2) {
   if (last1 - first1 < last2 - first2) {
     return false;
   }
   return std::equal(first2, last2, first1);
 }
 
-inline bool startsWith(const std::string &a, const std::string &b) {
-  return startsWith(std::begin(a), std::end(a), std::begin(b), std::end(b));
+inline bool starts_with(const std::string &a, const std::string &b) {
+  return starts_with(std::begin(a), std::end(a), std::begin(b), std::end(b));
 }
 
-inline bool startsWith(const char *a, const char *b) {
-  return startsWith(a, a + strlen(a), b, b + strlen(b));
+inline bool starts_with(const char *a, const char *b) {
+  return starts_with(a, a + strlen(a), b, b + strlen(b));
 }
 
 struct CaseCmp {
@@ -289,49 +192,49 @@ struct CaseCmp {
 };
 
 template <typename InputIterator1, typename InputIterator2>
-bool istartsWith(InputIterator1 first1, InputIterator1 last1,
-                 InputIterator2 first2, InputIterator2 last2) {
+bool istarts_with(InputIterator1 first1, InputIterator1 last1,
+                  InputIterator2 first2, InputIterator2 last2) {
   if (last1 - first1 < last2 - first2) {
     return false;
   }
   return std::equal(first2, last2, first1, CaseCmp());
 }
 
-inline bool istartsWith(const std::string &a, const std::string &b) {
-  return istartsWith(std::begin(a), std::end(a), std::begin(b), std::end(b));
+inline bool istarts_with(const std::string &a, const std::string &b) {
+  return istarts_with(std::begin(a), std::end(a), std::begin(b), std::end(b));
 }
 
 template <typename InputIt>
-bool istartsWith(InputIt a, size_t an, const char *b) {
-  return istartsWith(a, a + an, b, b + strlen(b));
+bool istarts_with(InputIt a, size_t an, const char *b) {
+  return istarts_with(a, a + an, b, b + strlen(b));
 }
 
-bool istartsWith(const char *a, const char *b);
+bool istarts_with(const char *a, const char *b);
 
 template <typename InputIterator1, typename InputIterator2>
-bool endsWith(InputIterator1 first1, InputIterator1 last1,
-              InputIterator2 first2, InputIterator2 last2) {
+bool ends_with(InputIterator1 first1, InputIterator1 last1,
+               InputIterator2 first2, InputIterator2 last2) {
   if (last1 - first1 < last2 - first2) {
     return false;
   }
   return std::equal(first2, last2, last1 - (last2 - first2));
 }
 
-inline bool endsWith(const std::string &a, const std::string &b) {
-  return endsWith(std::begin(a), std::end(a), std::begin(b), std::end(b));
+inline bool ends_with(const std::string &a, const std::string &b) {
+  return ends_with(std::begin(a), std::end(a), std::begin(b), std::end(b));
 }
 
 template <typename InputIterator1, typename InputIterator2>
-bool iendsWith(InputIterator1 first1, InputIterator1 last1,
-               InputIterator2 first2, InputIterator2 last2) {
+bool iends_with(InputIterator1 first1, InputIterator1 last1,
+                InputIterator2 first2, InputIterator2 last2) {
   if (last1 - first1 < last2 - first2) {
     return false;
   }
   return std::equal(first2, last2, last1 - (last2 - first2), CaseCmp());
 }
 
-inline bool iendsWith(const std::string &a, const std::string &b) {
-  return iendsWith(std::begin(a), std::end(a), std::begin(b), std::end(b));
+inline bool iends_with(const std::string &a, const std::string &b) {
+  return iends_with(std::begin(a), std::end(a), std::begin(b), std::end(b));
 }
 
 int strcompare(const char *a, const uint8_t *b, size_t n);
@@ -365,11 +268,11 @@ inline bool strieq(const char *a, const std::string &b) {
 }
 
 template <typename InputIt, size_t N>
-bool strieq_l(const char (&a)[N], InputIt b, size_t blen) {
+bool strieq_l(const char(&a)[N], InputIt b, size_t blen) {
   return strieq(a, N - 1, b, blen);
 }
 
-template <size_t N> bool strieq_l(const char (&a)[N], const std::string &b) {
+template <size_t N> bool strieq_l(const char(&a)[N], const std::string &b) {
   return strieq(a, N - 1, std::begin(b), b.size());
 }
 
@@ -399,10 +302,14 @@ inline bool streq(const char *a, const char *b) {
 }
 
 template <typename InputIt, size_t N>
-bool streq_l(const char (&a)[N], InputIt b, size_t blen) {
+bool streq_l(const char(&a)[N], InputIt b, size_t blen) {
   return streq(a, N - 1, b, blen);
 }
 
+template <size_t N> bool streq_l(const char(&a)[N], const std::string &b) {
+  return streq(a, N - 1, std::begin(b), b.size());
+}
+
 bool strifind(const char *a, const char *b);
 
 template <typename InputIt> void inp_strlower(InputIt first, InputIt last) {
@@ -579,7 +486,8 @@ bool select_h2(const unsigned char **out, unsigned char *outlen,
 // present in |in| of length inlen.  Returns true if identifier is
 // selected.
 bool select_protocol(const unsigned char **out, unsigned char *outlen,
-               const unsigned char *in, unsigned int inlen, std::vector<std::string> proto_list);
+                     const unsigned char *in, unsigned int inlen,
+                     std::vector<std::string> proto_list);
 
 // Returns default ALPN protocol list, which only contains supported
 // HTTP/2 protocol identifier.
@@ -659,9 +567,10 @@ int64_t parse_uint(const std::string &s);
 // Parses NULL terminated string |s| as unsigned integer and returns
 // the parsed integer casted to double.  If |s| ends with "s", the
 // parsed value's unit is a second.  If |s| ends with "ms", the unit
-// is millisecond.  If none of them are given, the unit is second.
-// This function returns std::numeric_limits<double>::infinity() if
-// error occurs.
+// is millisecond.  Similarly, it also supports 'm' and 'h' for
+// minutes and hours respectively.  If none of them are given, the
+// unit is second.  This function returns
+// std::numeric_limits<double>::infinity() if error occurs.
 double parse_duration_with_unit(const char *s);
 
 // Returns string representation of time duration |t|.  If t has
@@ -676,6 +585,9 @@ std::string duration_str(double t);
 // fractional digits follow.
 std::string format_duration(const std::chrono::microseconds &u);
 
+// Just like above, but this takes |t| as seconds.
+std::string format_duration(double t);
+
 // Creates "host:port" string using given |host| and |port|.  If
 // |host| is numeric IPv6 address (e.g., ::1), it is enclosed by "["
 // and "]".  If |port| is 80 or 443, port part is omitted.
@@ -704,6 +616,12 @@ uint32_t get_uint32(const uint8_t *data);
 // order and returns it in host byte order.
 uint64_t get_uint64(const uint8_t *data);
 
+// Reads mime types file (see /etc/mime.types), and stores extension
+// -> MIME type map in |res|.  This function returns 0 if it succeeds,
+// or -1.
+int read_mime_types(std::map<std::string, std::string> &res,
+                    const char *filename);
+
 } // namespace util
 
 } // namespace nghttp2
